==> fahrschein-0.1.0-sources.jar: org/zalando/fahrschein/ExponentialBackoffStrategy.java <==
package org.zalando.fahrschein;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class ExponentialBackoffStrategy {
    private static final Logger LOG = LoggerFactory.getLogger(ExponentialBackoffStrategy.class);

    public static final int DEFAULT_INITIAL_DELAY = 500;
    public static final double DEFAULT_BACKOFF_FACTOR = 1.5;
    public static final long DEFAULT_MAX_DELAY = 10 * 60 * 1000L;

    private final int initialDelay;
    private final double backoffFactor;
    private final long maxDelay;
    private final int maxRetries;

    public ExponentialBackoffStrategy() {
        this(DEFAULT_INITIAL_DELAY, DEFAULT_BACKOFF_FACTOR, DEFAULT_MAX_DELAY, -1);
    }

    public ExponentialBackoffStrategy(int initialDelay, double backoffFactor, long maxDelay, int maxRetries) {
        this.initialDelay = initialDelay;
        this.backoffFactor = backoffFactor;
        this.maxDelay = maxDelay;
        this.maxRetries = maxRetries;
    }

    private long calculateDelay(double count) {
        return Math.min((long)(initialDelay*Math.pow(backoffFactor, count)), maxDelay);
    }

    private void sleepForRetries(final int count) throws InterruptedException {
        final long delay = calculateDelay(count);
        LOG.info("Retry [{}], sleeping for [{}] milliseconds", count, delay);
        Thread.sleep(delay);
    }

    public <T> T call(final IOCallable<T> callable) throws ExponentialBackoffException, InterruptedException {
        return call(0, callable);
    }

    public <T> T call(final int initialCount, final IOCallable<T> callable) throws ExponentialBackoffException, InterruptedException{
        int count = initialCount;

        if (count > 0) {
            sleepForRetries(count);
        }

        while (true) {
            try {
                LOG.trace("Try [{}]", count);
                return callable.call();
            } catch (IOException e) {
                LOG.warn("Got [{}]", e.getClass().getSimpleName(), e);
                count++;

                if (maxRetries >= 0 && count > maxRetries) {
                    LOG.info("Maximum number of retries exceeded");
                    throw new ExponentialBackoffException(e);
                }
                sleepForRetries(count);
            }
        }
    }
}

==> fahrschein-0.2.1-sources.jar: org/zalando/fahrschein/ExponentialBackoffStrategy.java <==
package org.zalando.fahrschein;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.io.IOException;

import static com.google.common.base.Preconditions.checkState;

public class ExponentialBackoffStrategy implements BackoffStrategy {
    private static final Logger LOG = LoggerFactory.getLogger(ExponentialBackoffStrategy.class);

    public static final int DEFAULT_INITIAL_DELAY = 500;
    public static final double DEFAULT_BACKOFF_FACTOR = 1.5;
    public static final long DEFAULT_MAX_DELAY = 10 * 60 * 1000L;

    private final int initialDelay;
    private final double backoffFactor;
    private final long maxDelay;
    private final int maxRetries;

    public ExponentialBackoffStrategy() {
        this(DEFAULT_INITIAL_DELAY, DEFAULT_BACKOFF_FACTOR, DEFAULT_MAX_DELAY, -1);
    }

    public ExponentialBackoffStrategy(int initialDelay, double backoffFactor, long maxDelay, int maxRetries) {
        checkState(initialDelay > 0, "Initial delay should be bigger than 0");

        this.initialDelay = initialDelay;
        this.backoffFactor = backoffFactor;
        this.maxDelay = maxDelay;
        this.maxRetries = maxRetries;
    }

    private long calculateDelay(double count) {
        return Math.min((long)(initialDelay*Math.pow(backoffFactor, count)), maxDelay);
    }

    private void sleepForRetries(final int count) throws InterruptedException {
        final long delay = calculateDelay(count);
        LOG.info("Retry [{}], sleeping for [{}] milliseconds", count, delay);
        Thread.sleep(delay);
    }

    private void checkMaxRetries(final @Nullable IOException exception, final int count) throws BackoffException {
        if (maxRetries >= 0 && count > maxRetries) {
            LOG.info("Number of retries [{}] is higher than configured maximum [{}]", count, maxRetries);
            checkState(exception != null, "Exception should be present when retries are exceeded");
            throw new BackoffException(exception);
        }
    }

    @Override
    public <T> T call(final int initialExceptionCount, final @Nullable IOException initialException, final IOCallable<T> callable) throws BackoffException, InterruptedException{
        checkMaxRetries(initialException, initialExceptionCount);

        int count = initialExceptionCount;

        if (count > 0) {
            sleepForRetries(count);
        }

        while (true) {
            try {
                LOG.trace("Try [{}]", count);
                return callable.call();
            } catch (IOException e) {
                LOG.warn("Got [{}]", e.getClass().getSimpleName(), e);
                count++;

                checkMaxRetries(e, count);
                sleepForRetries(count);
            }
        }
    }
}
